"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .evaluationrecordstatus import EvaluationRecordStatus
from .llmevaluationerrortype import LLMEvaluationErrorType
from .ragcriteriaresult import RagCriteriaResult
from .ragrecord import RagRecord
from dataclasses_json import Undefined, dataclass_json
from gantry import utils
from typing import List, Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class RagEvaluationResult:
    r"""RagEvaluationResult stores both in-flight and completed evaluation result values.

    Completed evaluation results are mapped to EvaluationResults Dynamo model and stored in
    DynamoDB.

    TODO: Create class inheritance based on the in-flight status of the evaluation result.
    """
    criteria_results: List[RagCriteriaResult] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('criteria_results') }})
    record: RagRecord = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('record') }})
    record_id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('record_id') }})
    status: EvaluationRecordStatus = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('status') }})
    r"""An enumeration."""
    timestamp: float = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('timestamp') }})
    topics: List[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('topics') }})
    error_message: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('error_message'), 'exclude': lambda f: f is None }})
    error_type: Optional[LLMEvaluationErrorType] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('error_type'), 'exclude': lambda f: f is None }})
    r"""An enumeration."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http
from ...models.components import getragevaluationresultsresponseresponse as components_getragevaluationresultsresponseresponse
from enum import Enum
from typing import Optional, Union

class Two(str, Enum):
    NAN = 'nan'


@dataclasses.dataclass
class GetRagEvaluationResultsRequest:
    evaluation_run_id: str = dataclasses.field(metadata={'path_param': { 'field_name': 'evaluation_run_id', 'style': 'simple', 'explode': False }})
    criteria_id: Optional[str] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'criteria_id', 'style': 'form', 'explode': False }})
    limit: Optional[int] = dataclasses.field(default=10, metadata={'query_param': { 'field_name': 'limit', 'style': 'form', 'explode': False }})
    search: Optional[str] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'search', 'style': 'form', 'explode': False }})
    start: Optional[str] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'start', 'style': 'form', 'explode': False }})
    winner_version: Optional[Union[str, Two]] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'winner_version', 'style': 'form', 'explode': False }})
    



@dataclasses.dataclass
class GetRagEvaluationResultsResponse:
    content_type: str = dataclasses.field()
    r"""HTTP response content type for this operation"""
    raw_response: requests_http.Response = dataclasses.field()
    r"""Raw HTTP response; suitable for custom response parsing"""
    status_code: int = dataclasses.field()
    r"""HTTP response status code for this operation"""
    get_rag_evaluation_results_response_response: Optional[components_getragevaluationresultsresponseresponse.GetRagEvaluationResultsResponseResponse] = dataclasses.field(default=None)
    r"""OK"""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .criteriastats import CriteriaStats
from .criteriastatuscount import CriteriaStatusCount
from .recordstatuscount import RecordStatusCount
from .topicstats import TopicStats
from dataclasses_json import Undefined, dataclass_json
from gantry import utils
from typing import Dict, List, Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class EvalStats:
    all_criteria_stats: CriteriaStatusCount = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('all_criteria_stats') }})
    r"""Criteria status aggregated at record level"""
    record_status_count: RecordStatusCount = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('record_status_count') }})
    total_record_count: int = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('total_record_count') }})
    criteria_stats: Optional[Dict[str, CriteriaStats]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('criteria_stats'), 'exclude': lambda f: f is None }})
    ordered_criteria_stats: Optional[List[CriteriaStats]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('ordered_criteria_stats'), 'exclude': lambda f: f is None }})
    topic_stats: Optional[List[TopicStats]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('topic_stats'), 'exclude': lambda f: f is None }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class StepFormat(str, Enum):
    r"""An enumeration."""
    JSON = 'json'
    MARKDOWN = 'markdown'
    HTML = 'html'



from typing import Optional

from gantry.logger.utils import _init_gantry
from gantry.models.operations.get_rag_evaluation_results import (
    GetRagEvaluationResultsRequest,
)


class EvaluationReport:
    def __init__(self, evaluation_id: str, api_key: Optional[str] = None) -> None:
        self._sdk = _init_gantry(api_key)
        self._eval_id = evaluation_id

    def get_results(self):
        return self._sdk.evaluations.get_rag_evaluation_results(
            GetRagEvaluationResultsRequest(
                evaluation_run_id=self._eval_id, start=0, limit=10_000
            )
        ).get_rag_evaluation_results_response_response.data



import os
import json
import logging
from abc import ABC, abstractmethod
from queue import Queue
from typing import Dict, List, Optional

import requests
from gantry.logger.consumer import BatchConsumer
from gantry.logger.eval_reports import EvaluationReport
from gantry.logger.utils import _init_gantry
from gantry.models.components.createragevaluationrequest import (
    CreateRAGEvaluationRequest,
)
from gantry.models.components.logragrecordrequest import LogRagRecordRequest

logger = logging.getLogger(__name__)


class EvalReportAlreadyCreated(Exception):
    pass


class OutputLog(ABC):
    def __init__(self, **kwargs):
        pass

    @abstractmethod
    def open(self):
        """Start the output."""
        ...

    @abstractmethod
    def write(self, data: Dict):
        """Write a line of data to the output."""
        ...

    @abstractmethod
    def close(self):
        """Close the output."""
        ...


class FileLog(OutputLog):
    def __init__(self, filename: str, **kwargs):
        super().__init__(**kwargs)
        self.filename = filename
        self._f = None

    def open(self):
        """Open the file for appending"""
        self._f = open(self.filename, "a")

    def write(self, data):
        self._f.write(json.dumps(data))
        self._f.write("\n")

    def close(self):
        """Close the file"""
        self._f.close()


class EvalReportLog(OutputLog):
    def __init__(self, name: str, api_key: Optional[str] = None, **kwargs):
        super().__init__(**kwargs)
        self._records: List[Dict] = []
        self._name = name
        self._sdk = _init_gantry(api_key)
        self._eval_id = None
        self._report_created = False

    def open(self):
        pass

    def write(self, data):
        """Add the record to the list of records to be uploaded."""
        self._records.append(data)

    def close(self):
        """Create the evaluation run and upload the records."""
        if not self._records:
            return

        pre_upload = self._sdk.evaluations.create_evaluation_run_pre_upload()
        pending_id = pre_upload.evaluation_run_pre_upload_response.data.id
        presigned_url = pre_upload.evaluation_run_pre_upload_response.data.presigned_url

        requests.put(
            presigned_url,
            data="\n".join(json.dumps(record) for record in self._records),
            headers={"Content-Type": "text/plain; charset=utf-8"},
        )

        eval_run = self._sdk.evaluations.create_rag_evaluation_run(
            CreateRAGEvaluationRequest(
                concepts_mapping={"query": "query", "chat_history": "chat_history"},
                name=self._name,
                pending_dataset_id=pending_id,
            )
        )

        self._eval_id = eval_run.evaluation_run_response.data.id
        self._records = []
        return self._eval_id

    def create_evaluation_report(self):
        """Create the evaluation report."""
        if self._report_created:
            raise EvalReportAlreadyCreated(
                "An evaluation report has already been created. "
                "Please create a new EvalReportLogger to create a new report."
            )

        eval_report_id = self.close()
        self._report_created = True
        server_url, _ = self._sdk.sdk_configuration.get_server_details()
        print(
            "Evaluation report created! View it at "
            f"{server_url}/evaluations/{eval_report_id}"
        )
        return EvaluationReport(
            eval_report_id, api_key=self._sdk.sdk_configuration.security.api_key_auth
        )


class StreamLog(OutputLog):
    def __init__(
        self,
        source_name: str,
        api_key: Optional[str] = None,
        send_in_background: bool = True,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self._source_name = source_name
        self._sdk = _init_gantry(api_key)

        self.num_consumer_threads: int = 1
        if send_in_background:
            # We may need to override send_in_background=True if running in lambda.
            # Set sync behavior in case app is running in a lambda (scenario in which
            # we can't start a thread).
            # https://docs.aws.amazon.com/lambda/latest/dg/configuration-envvars.html
            send_in_background = os.environ.get("AWS_LAMBDA_FUNCTION_NAME") is None
        self.send_in_background = send_in_background
        self.queue: Queue = Queue()
        self.consumers = []

        if self.send_in_background:
            for _ in range(self.num_consumer_threads):
                consumer = BatchConsumer(self.queue, self.consumer_func)
                self.consumers.append(consumer)
                consumer.start()

    def consumer_func(self, batch):
        # Catch all errors and do not raise in order
        # for thread consumer to continue running.
        try:
            self._sdk.logs.log_rag_records(
                LogRagRecordRequest(records=batch, source_name=self._source_name)
            )
        except Exception as e:
            logger.error("Internal error sending batches: %s", e)

    def open(self):
        pass

    def write(self, data):
        """Push the record to the logs api."""
        if self.send_in_background:
            self.queue.put(data)
        else:
            self._sdk.logs.log_rag_records(
                LogRagRecordRequest(records=[data], source_name=self._source_name)
            )

    def close(self):
        for consumer in self.consumers:
            consumer.pause()
            try:
                consumer.join()
            except RuntimeError:
                # consumer thread has not started
                pass



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from gantry import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class GetLogsSummaryRequest:
    r"""Base Pydantic model to use when validating API request data instead
    of pydantic.BaseModel.
    This raises validation errors as GantryRequestValidationError so they can
    be caught and returned as 400s.
    """
    source_id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('source_id') }})
    end_time: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('end_time'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    relative_time_range: Optional[int] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('relative_time_range'), 'exclude': lambda f: f is None }})
    sample_percentage: Optional[int] = dataclasses.field(default=100, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('sample_percentage'), 'exclude': lambda f: f is None }})
    start_time: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('start_time'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .document import Document
from .stepformat import StepFormat
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from gantry import utils
from typing import List, Optional, Union

class Type(str, Enum):
    RETRIEVAL = 'retrieval'


@dataclasses.dataclass
class Two:
    pass


@dataclasses.dataclass
class Metadata:
    pass


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class RetrievalStep:
    documents: List[Document] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('documents') }})
    query: Union[str, Two] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('query') }})
    type: Type = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('type') }})
    format: Optional[StepFormat] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('format'), 'exclude': lambda f: f is None }})
    r"""An enumeration."""
    metadata: Optional[Metadata] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .stepformat import StepFormat
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from gantry import utils
from typing import Optional

class FunctionStepType(str, Enum):
    FUNCTION = 'function'


@dataclasses.dataclass
class Args:
    pass


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class FunctionStep:
    args: Args = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('args') }})
    name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    output: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('output') }})
    type: FunctionStepType = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('type') }})
    format: Optional[StepFormat] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('format'), 'exclude': lambda f: f is None }})
    r"""An enumeration."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class EvaluationCriteriaStatus(str, Enum):
    r"""An enumeration."""
    RUNNING = 'RUNNING'
    SUCCEEDED = 'SUCCEEDED'
    FAILED = 'FAILED'



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

import requests as requests_http
from .evaluations import Evaluations
from .logs import Logs
from .sdkconfiguration import SDKConfiguration
from gantry import utils
from gantry.models import components
from typing import Dict

class Gantry:
    evaluations: Evaluations
    logs: Logs

    sdk_configuration: SDKConfiguration

    def __init__(self,
                 api_key_auth: str ,
                 server_idx: int = None,
                 server_url: str = None,
                 url_params: Dict[str, str] = None,
                 client: requests_http.Session = None,
                 retry_config: utils.RetryConfig = None
                 ) -> None:
        """Instantiates the SDK configuring it with the provided parameters.
        
        :param api_key_auth: The api_key_auth required for authentication
        :type api_key_auth: Union[str,Callable[[], str]]
        :param server_idx: The index of the server to use for all operations
        :type server_idx: int
        :param server_url: The server URL to use for all operations
        :type server_url: str
        :param url_params: Parameters to optionally template the server URL with
        :type url_params: Dict[str, str]
        :param client: The requests.Session HTTP client to use for all operations
        :type client: requests_http.Session
        :param retry_config: The utils.RetryConfig to use globally
        :type retry_config: utils.RetryConfig
        """
        if client is None:
            client = requests_http.Session()
        
        security = components.Security(api_key_auth = api_key_auth)
        
        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        self.sdk_configuration = SDKConfiguration(client, security, server_url, server_idx, retry_config=retry_config)
       
        self._init_sdks()
    
    def _init_sdks(self):
        self.evaluations = Evaluations(self.sdk_configuration)
        self.logs = Logs(self.sdk_configuration)
    


"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .criteriafeedback import CriteriaFeedback
from .evaluationcriteriastatus import EvaluationCriteriaStatus
from .passfailunsurerubricscalevalues import PassFailUnsureRubricScaleValues
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from gantry import utils
from typing import List, Optional

class RagCriteriaStepType(str, Enum):
    r"""An enumeration."""
    QUERY = 'query'
    CHAT_HISTORY = 'chat_history'
    FUNCTION = 'function'
    DOCUMENTS = 'documents'
    OUTPUT = 'output'


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class RagCriteriaResult:
    r"""Stores an evaluation criteria generation's parsed score and
    reasoning for a particular RAG step.
    """
    criteria_eval_instruction: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('criteria_eval_instruction') }})
    criteria_id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('criteria_id') }})
    criteria_name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('criteria_name') }})
    error_message: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('error_message'), 'exclude': lambda f: f is None }})
    error_type: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('error_type'), 'exclude': lambda f: f is None }})
    feedback: Optional[List[CriteriaFeedback]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('feedback'), 'exclude': lambda f: f is None }})
    id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id'), 'exclude': lambda f: f is None }})
    include_input_step: Optional[bool] = dataclasses.field(default=False, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('include_input_step'), 'exclude': lambda f: f is None }})
    include_output_step: Optional[bool] = dataclasses.field(default=False, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('include_output_step'), 'exclude': lambda f: f is None }})
    reasoning: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('reasoning'), 'exclude': lambda f: f is None }})
    relevant_step: Optional[RagCriteriaStepType] = dataclasses.field(default=RagCriteriaStepType.OUTPUT, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('relevant_step'), 'exclude': lambda f: f is None }})
    score: Optional[PassFailUnsureRubricScaleValues] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('score'), 'exclude': lambda f: f is None }})
    r"""An enumeration."""
    status: Optional[EvaluationCriteriaStatus] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('status'), 'exclude': lambda f: f is None }})
    r"""An enumeration."""
    steps: Optional[List[int]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('steps'), 'exclude': lambda f: f is None }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from .evalcriteria import EvalCriteria
from .evalstats import EvalStats
from .evaluationrunjobstatus import EvaluationRunJobStatus
from .getlogssummaryrequest import GetLogsSummaryRequest
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from gantry import utils
from typing import Dict, List, Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class EvaluationRun:
    created_at: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('created_at'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    evaluation_job_status: EvaluationRunJobStatus = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('evaluation_job_status') }})
    r"""An enumeration."""
    id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id') }})
    organization_id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('organization_id') }})
    column_mapping: Optional[Dict[str, str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('column_mapping'), 'exclude': lambda f: f is None }})
    completed_at: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('completed_at'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    created_by: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('created_by'), 'exclude': lambda f: f is None }})
    is_draft: Optional[bool] = dataclasses.field(default=False, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('is_draft'), 'exclude': lambda f: f is None }})
    llm_evaluation_criterias: Optional[List[EvalCriteria]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('llm_evaluation_criterias'), 'exclude': lambda f: f is None }})
    logs_query: Optional[GetLogsSummaryRequest] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('logs_query'), 'exclude': lambda f: f is None }})
    r"""Base Pydantic model to use when validating API request data instead
    of pydantic.BaseModel.
    This raises validation errors as GantryRequestValidationError so they can
    be caught and returned as 400s.
    """
    name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name'), 'exclude': lambda f: f is None }})
    num_records: Optional[int] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('num_records'), 'exclude': lambda f: f is None }})
    num_tokens: Optional[int] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('num_tokens'), 'exclude': lambda f: f is None }})
    pending_dataset_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('pending_dataset_id'), 'exclude': lambda f: f is None }})
    rag_stats: Optional[EvalStats] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('rag_stats'), 'exclude': lambda f: f is None }})
    raw_dataset_path: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('raw_dataset_path'), 'exclude': lambda f: f is None }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from .create_evaluation_run_pre_upload import *
from .create_rag_evaluation_run import *
from .get_rag_evaluation_results import *
from .log_rag_records import *

__all__ = ["CreateEvaluationRunPreUploadResponse","CreateRagEvaluationRunResponse","GetRagEvaluationResultsRequest","GetRagEvaluationResultsResponse","LogRagRecordsResponse","Two"]



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class LLMEvaluationCriteriaType(str, Enum):
    r"""An enumeration."""
    DEFAULT = 'DEFAULT'
    CUSTOM = 'CUSTOM'
    PROJECTION = 'PROJECTION'



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .stepformat import StepFormat
from dataclasses_json import Undefined, dataclass_json
from gantry import utils
from typing import List, Optional


@dataclasses.dataclass
class FunctionCall:
    pass


@dataclasses.dataclass
class ToolCalls:
    pass


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Message:
    type: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('type') }})
    content: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('content'), 'exclude': lambda f: f is None }})
    format: Optional[StepFormat] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('format'), 'exclude': lambda f: f is None }})
    r"""An enumeration."""
    function_call: Optional[FunctionCall] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('function_call'), 'exclude': lambda f: f is None }})
    tool_calls: Optional[List[ToolCalls]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('tool_calls'), 'exclude': lambda f: f is None }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class LLMEvaluationErrorType(str, Enum):
    r"""An enumeration."""
    USER = 'USER'
    SYSTEM = 'SYSTEM'



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from gantry import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class TopicStats:
    topic: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('topic') }})
    count: Optional[int] = dataclasses.field(default=0, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('count'), 'exclude': lambda f: f is None }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .criteriastatuscount import CriteriaStatusCount
from dataclasses_json import Undefined, dataclass_json
from gantry import utils


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CriteriaStats:
    criteria_eval_instruction: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('criteria_eval_instruction') }})
    criteria_id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('criteria_id') }})
    criteria_name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('criteria_name') }})
    score_count: CriteriaStatusCount = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('score_count') }})
    r"""Criteria status aggregated at record level"""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .logragrecordresponse import LogRagRecordResponse
from dataclasses_json import Undefined, dataclass_json
from gantry import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class LogRagRecordResponseResponse:
    data: Optional[LogRagRecordResponse] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('data'), 'exclude': lambda f: f is None }})
    r"""Base Pydantic model to use when validating API request data instead
    of pydantic.BaseModel.
    This raises validation errors as GantryRequestValidationError so they can
    be caught and returned as 400s.
    """
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .organizationtier import OrganizationTier
from dataclasses_json import Undefined, dataclass_json
from gantry import utils
from typing import List, Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class OrgUser:
    email: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('email') }})
    id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id') }})
    auth0_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('auth0_id'), 'exclude': lambda f: f is None }})
    name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name'), 'exclude': lambda f: f is None }})
    org_ids: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('org_ids'), 'exclude': lambda f: f is None }})
    org_name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('org_name'), 'exclude': lambda f: f is None }})
    org_tier: Optional[OrganizationTier] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('org_tier'), 'exclude': lambda f: f is None }})
    r"""Represents the different tiers an organization can have."""
    




import atexit
import collections
import dataclasses
import logging
from enum import Enum
from typing import Any, Dict, List, Optional, Sequence
from uuid import UUID

from gantry.logger.eval_logger import EvalLogger

try:
    from langchain.callbacks.base import BaseCallbackHandler
    from langchain.schema import LLMResult
    from langchain_core.agents import AgentAction, AgentFinish
    from langchain_core.documents import Document
    from langchain_core.load.serializable import Serializable
    from langchain_core.messages import BaseMessage
except ImportError:
    raise RuntimeError("Please run `pip install langchain` to use this module.")

from pydantic import BaseModel
from pydantic.version import VERSION as PYDANTIC_VERSION

logger = logging.getLogger(__name__)


class GantryCallbackHandler(BaseCallbackHandler):
    _inputs: Dict

    def __init__(
        self,
        logger: EvalLogger,
        query_name: str = "query",
        chat_history_name: str = "chat_history",
    ) -> None:
        super().__init__()

        self.reset()

        self._logger = logger
        self._query_name = query_name
        self._chat_history_name = chat_history_name

        atexit.register(self.close)

        self._usage = collections.Counter(
            [
                "prompt_tokens",
                "completion_tokens",
                "total_tokens",
            ]
        )

    def on_chain_start(
        self, serialized: Dict[str, Any], inputs: Dict[str, Any], **kwargs: Any
    ) -> Any:
        logger.debug("on_chain_start: %s; %s", serialized, inputs)

        # a chain contains many sub-chains and will call on_chain_start many
        # times during a chain invocation
        if self._inputs is None:
            if isinstance(inputs, str):
                inputs = {"query": inputs}
            self._inputs = inputs
            self._metadata["serialized_langchain"] = serialized

    def on_chain_end(
        self,
        outputs: Dict[str, Any],
        *,
        run_id: UUID,
        parent_run_id: Optional[UUID] = None,
        **kwargs: Any,
    ) -> Any:
        """Run when chain ends running."""
        logger.debug("on_chain_end: %s", outputs)

        # if we're not in a subchain, log the record
        if not parent_run_id:
            try:
                # langchain type annotation lies
                self._outputs = outputs.dict()  # type: ignore
            except Exception:
                if isinstance(outputs, str):
                    outputs = {"content": outputs}
                self._outputs = outputs

            self.end()

    def on_chat_model_start(
        self,
        serialized: Dict[str, Any],
        messages: List[List[BaseMessage]],
        *,
        run_id: UUID,
        parent_run_id: Optional[UUID] = None,
        tags: Optional[List[str]] = None,
        metadata: Optional[Dict[str, Any]] = None,
        **kwargs: Any,
    ) -> Any:
        logger.debug("on_chat_model_start: %s; %s", serialized, messages)

        self._curr_messages = messages

    def on_llm_start(
        self, serialized: Dict[str, Any], prompts: List[str], **kwargs: Any
    ) -> Any:
        logger.debug("on_llm_start: %s; %s", serialized, prompts)

        self._curr_prompts = prompts

    def on_llm_end(self, response: LLMResult, **kwargs: Any) -> Any:
        """Run when LLM ends running."""
        logger.debug("on_llm_end: %s", response)

        i = 0
        for generations in response.generations:
            for generation in generations:
                self._steps.append(
                    {
                        "type": "llm",
                        # "prompt": self._curr_prompts[i],
                        # TODO: not super sure if one on_chat_model_start maps to
                        # one on_llm_end
                        "messages": self._curr_messages[i],
                        "response": generation.text,
                    }
                )
                i += 1

        self._usage.update(response.llm_output.get("token_usage", {}))  # type: ignore

    def on_text(self, text: str, **kwargs: Any) -> Any:
        logger.debug("on_text: %s", text)

    def on_tool_start(
        self, serialized: Dict[str, Any], input_str: str, **kwargs: Any
    ) -> Any:
        logger.debug("on_tool_start: %s; %s", serialized, input_str)

        self._curr_input = input_str

    def on_tool_end(
        self,
        output: str,
        *,
        run_id: UUID,
        parent_run_id: Optional[UUID] = None,
        **kwargs: Any,
    ) -> Any:
        self._steps.append(
            {
                "type": "tool",
                "args": [self._curr_input],
                "output": output,
            }
        )

    def on_agent_action(self, action: AgentAction, **kwargs: Any) -> Any:
        logger.debug("on_agent_action: %s", action)

    def on_agent_finish(self, finish: AgentFinish, **kwargs: Any) -> Any:
        logger.debug("on_agent_finish: %s", finish)

    def on_retriever_start(
        self,
        serialized: Dict[str, Any],
        query: str,
        *,
        run_id: UUID,
        parent_run_id: Optional[UUID] = None,
        tags: Optional[List[str]] = None,
        metadata: Optional[Dict[str, Any]] = None,
        **kwargs: Any,
    ) -> Any:
        logger.debug("on_retriever_start: %s; %s", serialized, query)
        self._curr_query = query

    def on_retriever_end(
        self,
        documents: Sequence[Document],
        *,
        run_id: UUID,
        parent_run_id: Optional[UUID] = None,
        **kwargs: Any,
    ) -> Any:
        logger.debug("on_retriever_end: %s", documents)
        self._steps.append(
            {
                "type": "retrieval",
                "query": self._curr_query,
                "documents": [self._format_document(d) for d in documents],
            }
        )

    def _format_document(self, doc: Document):
        return {"content": doc.page_content}

    def get_steps(self):
        return self._steps

    def get_usage(self):
        return self._usage

    def get_inputs(self):
        inputs = self._inputs.copy()
        inputs["query"] = self._inputs.pop(self._query_name, "")
        inputs["chat_history"] = self._inputs.pop(self._chat_history_name, [])

        return inputs

    def get_record(self):
        return _json_encode(
            {
                "inputs": self._inputs,
                "outputs": self._outputs,
                "steps": self._steps,
                "metadata": self._metadata,
            }
        )

    def reset(self):
        self._curr_prompts = []
        self._curr_messages = []
        self._curr_query = None
        self._curr_input = None
        self._inputs = None
        self._outputs = None
        self._steps = []
        self._metadata = {}

    def end(self):
        self._logger._write_record(self.get_record())

        self.reset()

    def close(self):
        self._logger.close()


def _json_encode(obj):
    if isinstance(obj, Serializable):
        return obj.dict()
    if isinstance(obj, BaseModel):
        if PYDANTIC_VERSION.startswith("2."):
            obj_dict = obj.model_dump()
        else:
            obj_dict = obj.dict()
        if "__root__" in obj_dict:
            obj_dict = obj_dict["__root__"]
        return _json_encode(obj_dict)
    if dataclasses.is_dataclass(obj):
        obj_dict = dataclasses.asdict(obj)
        return _json_encode(obj_dict)
    if isinstance(obj, Enum):
        return obj.value
    if isinstance(obj, dict):
        encoded_dict = {}
        for key, value in obj.items():
            encoded_key = _json_encode(key)
            encoded_value = _json_encode(value)
            encoded_dict[encoded_key] = encoded_value
        return encoded_dict
    if isinstance(obj, (list, set, frozenset, tuple)):
        encoded_list = []
        for item in obj:
            encoded_list.append(_json_encode(item))
        return encoded_list

    return obj



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from .createragevaluationrequest import *
from .criteriafeedback import *
from .criteriaresultcapture import *
from .criteriastats import *
from .criteriastatuscount import *
from .customstep import *
from .document import *
from .evalcriteria import *
from .evalstats import *
from .evaluationcriteriastatus import *
from .evaluationrecordstatus import *
from .evaluationrun import *
from .evaluationrunjobstatus import *
from .evaluationrunpreupload import *
from .evaluationrunpreuploadresponse import *
from .evaluationrunresponse import *
from .functionstep import *
from .getlogssummaryrequest import *
from .getragevaluationresultsresponse import *
from .getragevaluationresultsresponseresponse import *
from .llmevaluationcriteriatype import *
from .llmevaluationerrortype import *
from .llmstep import *
from .logragrecordrequest import *
from .logragrecordresponse import *
from .logragrecordresponseresponse import *
from .message import *
from .organizationtier import *
from .orguser import *
from .passfailunsurerubricscalevalues import *
from .ragcriteriaresult import *
from .ragevaluationresult import *
from .ragrecord import *
from .recordstatuscount import *
from .retrievalstep import *
from .security import *
from .stepformat import *
from .summarynotesresponse import *
from .topicstats import *

__all__ = ["Args","CreateRAGEvaluationRequest","CriteriaFeedback","CriteriaResultCapture","CriteriaStats","CriteriaStatusCount","CustomStep","CustomStepType","Document","EvalCriteria","EvalStats","EvaluationCriteriaStatus","EvaluationRecordStatus","EvaluationRun","EvaluationRunJobStatus","EvaluationRunPreUpload","EvaluationRunPreUploadResponse","EvaluationRunResponse","FunctionCall","FunctionStep","FunctionStepType","GetLogsSummaryRequest","GetRagEvaluationResultsResponse","GetRagEvaluationResultsResponseResponse","Inputs","LLMEvaluationCriteriaType","LLMEvaluationErrorType","LLMStep","LLMStepType","LogRagRecordRequest","LogRagRecordResponse","LogRagRecordResponseResponse","Message","Metadata","OrgUser","OrganizationTier","Outputs","Params","PassFailUnsureRubricScaleValues","RagCriteriaResult","RagCriteriaStepType","RagEvaluationResult","RagRecord","RagRecordMetadata","RecordStatusCount","Response2","RetrievalStep","Security","StepFormat","SummaryNotesResponse","ToolCalls","TopicStats","Two","Type"]



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .criteriaresultcapture import CriteriaResultCapture
from dataclasses_json import Undefined, dataclass_json
from gantry import utils


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CriteriaFeedback:
    comment: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('comment') }})
    current: CriteriaResultCapture = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('current') }})
    r"""CriteriaResultCapture stores current and update values in CriteriaFeedback"""
    id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id') }})
    update: CriteriaResultCapture = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('update') }})
    r"""CriteriaResultCapture stores current and update values in CriteriaFeedback"""
    updated_at: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('updated_at') }})
    updated_by: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('updated_by') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .getlogssummaryrequest import GetLogsSummaryRequest
from dataclasses_json import Undefined, dataclass_json
from gantry import utils
from typing import Dict, List, Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CreateRAGEvaluationRequest:
    r"""Base Pydantic model to use when validating API request data instead
    of pydantic.BaseModel.
    This raises validation errors as GantryRequestValidationError so they can
    be caught and returned as 400s.
    """
    concepts_mapping: Dict[str, str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('concepts_mapping') }})
    name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    copy_dataset_from_run_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('copy_dataset_from_run_id'), 'exclude': lambda f: f is None }})
    custom_criteria_ids: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('custom_criteria_ids'), 'exclude': lambda f: f is None }})
    logs_query: Optional[GetLogsSummaryRequest] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('logs_query'), 'exclude': lambda f: f is None }})
    r"""Base Pydantic model to use when validating API request data instead
    of pydantic.BaseModel.
    This raises validation errors as GantryRequestValidationError so they can
    be caught and returned as 400s.
    """
    pending_dataset_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('pending_dataset_id'), 'exclude': lambda f: f is None }})
    prompt_version_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('prompt_version_id'), 'exclude': lambda f: f is None }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class OrganizationTier(str, Enum):
    r"""Represents the different tiers an organization can have."""
    INTERNAL = 'internal'
    POC = 'poc'
    ENTERPRISE = 'enterprise'
    TEAM = 'team'



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .llmevaluationcriteriatype import LLMEvaluationCriteriaType
from dataclasses_json import Undefined, dataclass_json
from gantry import utils


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class EvalCriteria:
    r"""Criteria that was used in running an evaluation
    These are stored on the evaluation object itself
    so that we know what criteria was used to run the evaluation. This
    is to future proof the eval in case the criteria is changed after the eval
    """
    criteria_type: LLMEvaluationCriteriaType = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('criteria_type') }})
    r"""An enumeration."""
    id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id') }})
    name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .customstep import CustomStep
from .functionstep import FunctionStep
from .llmstep import LLMStep
from .retrievalstep import RetrievalStep
from .stepformat import StepFormat
from dataclasses_json import Undefined, dataclass_json
from gantry import utils
from typing import Dict, List, Optional, Union


@dataclasses.dataclass
class Inputs:
    pass


@dataclasses.dataclass
class Outputs:
    pass


@dataclasses.dataclass
class RagRecordMetadata:
    pass


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class RagRecord:
    inputs: Inputs = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('inputs') }})
    outputs: Outputs = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('outputs') }})
    steps: List[Union[RetrievalStep, LLMStep, FunctionStep, CustomStep]] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('steps') }})
    format: Optional[StepFormat] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('format'), 'exclude': lambda f: f is None }})
    r"""An enumeration."""
    labels: Optional[Dict[str, str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('labels'), 'exclude': lambda f: f is None }})
    metadata: Optional[RagRecordMetadata] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    session_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('session_id'), 'exclude': lambda f: f is None }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .stepformat import StepFormat
from dataclasses_json import Undefined, dataclass_json
from gantry import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Document:
    content: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('content') }})
    format: Optional[StepFormat] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('format'), 'exclude': lambda f: f is None }})
    r"""An enumeration."""
    title: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('title'), 'exclude': lambda f: f is None }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from gantry import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CriteriaStatusCount:
    r"""Criteria status aggregated at record level"""
    error: Optional[int] = dataclasses.field(default=0, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('ERROR'), 'exclude': lambda f: f is None }})
    fail: Optional[int] = dataclasses.field(default=0, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('FAIL'), 'exclude': lambda f: f is None }})
    feedback: Optional[int] = dataclasses.field(default=0, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('FEEDBACK'), 'exclude': lambda f: f is None }})
    not_applicable: Optional[int] = dataclasses.field(default=0, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('NOT_APPLICABLE'), 'exclude': lambda f: f is None }})
    pass_: Optional[int] = dataclasses.field(default=0, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('PASS'), 'exclude': lambda f: f is None }})
    unsure: Optional[int] = dataclasses.field(default=0, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('UNSURE'), 'exclude': lambda f: f is None }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class PassFailUnsureRubricScaleValues(str, Enum):
    r"""An enumeration."""
    PASS = 'PASS'
    FAIL = 'FAIL'
    UNSURE = 'UNSURE'
    NOT_APPLICABLE = 'NOT_APPLICABLE'
    ERROR = 'ERROR'



import atexit
from typing import Dict, List, Optional, TypedDict

from gantry.logger.log_location import EvalReportLog, FileLog, OutputLog, StreamLog


class Record(TypedDict):
    inputs: Dict
    outputs: Dict
    steps: List[Dict]


class EvalLogger:
    _record: Record

    def __init__(self, log_location: OutputLog) -> None:
        self._output = log_location
        self._output.open()

        atexit.register(self.close)

    def start_record(
        self,
        query: str,
        chat_history: Optional[List[Dict]] = None,
        session_id: Optional[str] = None,
        **kwargs,
    ):
        if not isinstance(query, str):
            raise TypeError(f"query must be a string, not {type(query)}")
        if not isinstance(chat_history, list) and chat_history is not None:
            raise TypeError(
                f"chat_history must be a list of dicts, not {type(chat_history)}"
            )

        self._record = {"inputs": {}, "outputs": {}, "steps": []}

        inputs: Dict = {"query": query}
        if chat_history:
            inputs["chat_history"] = chat_history
        if session_id:
            self._record["session_id"] = session_id
        inputs.update(kwargs)

        self.add_inputs(**inputs)

    def add_inputs(self, **kwargs):
        self._record["inputs"].update(kwargs)

    def add_outputs(self, **kwargs):
        self._record["outputs"].update(kwargs)

    def add_retrieval_step(self, query: str, documents: List[Dict]):
        self._record["steps"].append(
            {
                "type": "retrieval",
                "query": query,
                "documents": documents,
            }
        )

    def add_function_step(self, name: str, args: Dict, output: str):
        self._record["steps"].append(
            {
                "type": "function",
                "name": name,
                "args": args,
                "output": output,
            }
        )

    def add_llm_step(
        self, messages: List[Dict], response: str, params: Optional[Dict] = None
    ):
        self._record["steps"].append(
            {
                "type": "llm",
                "messages": list(messages),  # make a copy
                "response": response,
                "params": params,
            }
        )

    def add_custom_step(self, data: Dict):
        self._record["steps"].append(
            {
                "type": "custom",
                "data": data,
            }
        )

    def end_record(self, response: str, **kwargs):
        if not isinstance(response, str):
            raise TypeError(f"response must be a string, not {type(response)}")

        self.add_outputs(response=response, **kwargs)

        self._write_record(self._record)  # type: ignore

        self._record = None  # type: ignore

    def _write_record(self, record: Dict):
        self._output.write(record)

    def close(self):
        self._output.close()


class FileLogger(EvalLogger):
    _output: FileLog

    def __init__(self, path: str) -> None:
        super().__init__(FileLog(path))


class EvalReportLogger(EvalLogger):
    _output: EvalReportLog

    def __init__(self, name: str, api_key: Optional[str] = None) -> None:
        super().__init__(EvalReportLog(name=name, api_key=api_key))

    def create_evaluation_report(self):
        """Creates the evaluation report"""
        return self._output.create_evaluation_report()


class StreamLogger(EvalLogger):
    _output: StreamLog

    def __init__(
        self,
        source_name: str,
        api_key: Optional[str] = None,
        send_in_background: bool = True,
    ) -> None:
        super().__init__(
            StreamLog(
                source_name=source_name,
                api_key=api_key,
                send_in_background=send_in_background,
            )
        )



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .ragrecord import RagRecord
from dataclasses_json import Undefined, dataclass_json
from gantry import utils
from typing import List


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class LogRagRecordRequest:
    r"""Base Pydantic model to use when validating API request data instead
    of pydantic.BaseModel.
    This raises validation errors as GantryRequestValidationError so they can
    be caught and returned as 400s.
    """
    records: List[RagRecord] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('records') }})
    source_name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('source_name') }})
    




import setuptools

version = "0.0.2a3"


install_requires = [
    "pydantic",
    "certifi>=2023.7.22",
    "charset-normalizer>=3.2.0",
    "dataclasses-json>=0.6.1",
    "idna>=3.4",
    "jsonpath-python>=1.0.6 ",
    "marshmallow>=3.19.0",
    "mypy-extensions>=1.0.0",
    "packaging>=23.1",
    "python-dateutil>=2.8.2",
    "requests>=2.31.0",
    "six>=1.16.0",
    "typing-inspect>=0.9.0",
    "typing_extensions>=4.7.1",
    "urllib3>=1.26.18",
]


# https://packaging.python.org/tutorials/packaging-projects/
setuptools.setup(
    name="gantry-logger",
    version=version,
    install_requires=install_requires,
    include_package_data=True,
    author="Gantry Systems, Inc.",
    author_email="oss@gantry.io",
    description="Gantry Python Library",
    long_description="",
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages("src"),
    package_dir={"": "src"},
    license="Apache Software License v2",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
)



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class EvaluationRunJobStatus(str, Enum):
    r"""An enumeration."""
    CREATED = 'CREATED'
    PENDING = 'PENDING'
    RUNNING = 'RUNNING'
    COMPLETED = 'COMPLETED'
    FAILED = 'FAILED'
    PARTIAL = 'PARTIAL'
    CANCELLED = 'CANCELLED'



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from .sdkconfiguration import SDKConfiguration
from gantry import utils
from gantry.models import components, errors, operations
from typing import Optional

class Evaluations:
    sdk_configuration: SDKConfiguration

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        self.sdk_configuration = sdk_config
        
    
    
    def create_evaluation_run_pre_upload(self) -> operations.CreateEvaluationRunPreUploadResponse:
        r"""Generate a pre-signed URL to upload an evaluation dataset"""
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/api/v2/evaluation_runs/pre-upload'
        headers = {}
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        
        if callable(self.sdk_configuration.security):
            client = utils.configure_security_client(self.sdk_configuration.client, self.sdk_configuration.security())
        else:
            client = utils.configure_security_client(self.sdk_configuration.client, self.sdk_configuration.security)
        
        http_res = client.request('POST', url, headers=headers)
        content_type = http_res.headers.get('Content-Type')
        
        res = operations.CreateEvaluationRunPreUploadResponse(status_code=http_res.status_code, content_type=content_type, raw_response=http_res)
        
        if http_res.status_code == 200:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, Optional[components.EvaluationRunPreUploadResponse])
                res.evaluation_run_pre_upload_response = out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)

        return res

    
    
    def create_rag_evaluation_run(self, request: components.CreateRAGEvaluationRequest) -> operations.CreateRagEvaluationRunResponse:
        r"""Create a new RAG evaluation and kick off the workflow. Returns the created evaluation run"""
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/api/v2/rag_evaluation_runs'
        headers = {}
        req_content_type, data, form = utils.serialize_request_body(request, components.CreateRAGEvaluationRequest, "request", False, False, 'json')
        if req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        if data is None and form is None:
            raise Exception('request body is required')
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        
        if callable(self.sdk_configuration.security):
            client = utils.configure_security_client(self.sdk_configuration.client, self.sdk_configuration.security())
        else:
            client = utils.configure_security_client(self.sdk_configuration.client, self.sdk_configuration.security)
        
        http_res = client.request('POST', url, data=data, files=form, headers=headers)
        content_type = http_res.headers.get('Content-Type')
        
        res = operations.CreateRagEvaluationRunResponse(status_code=http_res.status_code, content_type=content_type, raw_response=http_res)
        
        if http_res.status_code == 200:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, Optional[components.EvaluationRunResponse])
                res.evaluation_run_response = out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)

        return res

    
    
    def get_rag_evaluation_results(self, request: operations.GetRagEvaluationResultsRequest) -> operations.GetRagEvaluationResultsResponse:
        r"""Get a list of evaluation run results"""
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = utils.generate_url(operations.GetRagEvaluationResultsRequest, base_url, '/api/v2/rag_evaluation_runs/{evaluation_run_id}/results', request)
        headers = {}
        query_params = utils.get_query_params(operations.GetRagEvaluationResultsRequest, request)
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        
        if callable(self.sdk_configuration.security):
            client = utils.configure_security_client(self.sdk_configuration.client, self.sdk_configuration.security())
        else:
            client = utils.configure_security_client(self.sdk_configuration.client, self.sdk_configuration.security)
        
        http_res = client.request('GET', url, params=query_params, headers=headers)
        content_type = http_res.headers.get('Content-Type')
        
        res = operations.GetRagEvaluationResultsResponse(status_code=http_res.status_code, content_type=content_type, raw_response=http_res)
        
        if http_res.status_code == 200:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, Optional[components.GetRagEvaluationResultsResponseResponse])
                res.get_rag_evaluation_results_response_response = out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)

        return res

    


import os
from typing import Optional

from gantry.sdk import Gantry


def _init_gantry(api_key: Optional[str] = None) -> Gantry:
    api_key = api_key or os.environ.get("GANTRY_API_KEY")
    if not api_key:
        raise ValueError(
            "You must provide an API key to use this feature. "
            "You can provide one as an argument or "
            "set the GANTRY_API_KEY environment variable."
        )
    return Gantry(
        api_key_auth=api_key,
        server_url=os.environ.get("GANTRY_LOGS_LOCATION"),  # type: ignore
    )
